package SPVM::Error::MethodCallNotPermitted;

1;

=head1 Name

SPVM::Error::MethodCallNotPermitted - Method Call Not Permitted Error

=head1 Usage

  use Error::MethodCallNotPermitted;

=head1 Description

Error::MethodCallNotPermitted class in L<SPVM> represents errors the method is not permitted to be call.

=head1 Super Class

L<Error|SPVM::Error>

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
