package SPVM::Callback::Map;



1;

=head1 Name

SPVM::Callback::Map - A Callback for The map Method

=head1 Description

The Callback::Map interface in L<SPVM> has interface methods for L<Fn#map|SPVM::Fn/"map">.

=head1 Usage

  interface Callback::Map;

=head1 Interface Methods

C<required method : object ($element : object);>

Returns an element after operating the element $element.

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

